=== modified file 'lib/lp/soyuz/adapters/notification.py'
--- lib/lp/soyuz/adapters/notification.py	2011-06-01 05:45:53 +0000
+++ lib/lp/soyuz/adapters/notification.py	2011-06-03 07:50:12 +0000
@@ -33,6 +33,7 @@
     ascii_smash,
     guess as guess_encoding,
     )
+from lp.services.mail.sendmail import format_address_for_person
 
 
 def reject_changes_file(blamer, changes_file_path, changes, archive,
@@ -62,7 +63,8 @@
     }
     template = get_template(archive, 'rejected')
     body = template % information
-    to_addrs = get_recipients(blamer, archive, distroseries, changes, logger)
+    to_addrs = get_recipients(
+        blamer, archive, distroseries, logger, changes=changes)
     logger.debug("Sending rejection email.")
     send_mail(None, archive, to_addrs, subject, body, False, logger=logger)
 
@@ -169,7 +171,8 @@
         return
 
     recipients = get_recipients(
-        blamer, archive, distroseries, changes, logger)
+        blamer, archive, distroseries, logger, changes=changes, spr=spr,
+        bprs=bprs)
 
     # There can be no recipients if none of the emails are registered
     # in LP.
@@ -200,7 +203,7 @@
         subject = calculate_subject(
             spr, bprs, customfiles, archive, distroseries, pocket, action)
         body = assemble_body(
-            blamer, spr, archive, distroseries, summarystring, changes,
+            blamer, spr, bprs, archive, distroseries, summarystring, changes,
             action)
         send_mail(
             spr, archive, recipients, subject, body, dry_run,
@@ -210,6 +213,8 @@
 
     build_and_send_mail(action, recipients)
 
+    (changesfile, date, from_addr, maintainer) = fetch_information(
+        spr, bprs, changes)
     # If we're sending an acceptance notification for a non-PPA upload,
     # announce if possible. Avoid announcing backports, binary-only
     # security uploads, or autosync uploads.
@@ -218,8 +223,7 @@
         and pocket != PackagePublishingPocket.BACKPORTS
         and not (pocket == PackagePublishingPocket.SECURITY and spr is None)
         and not is_auto_sync_upload(
-            spr, bprs, pocket, changes['Changed-By'])):
-        from_addr = sanitize_string(changes['Changed-By'])
+            spr, bprs, pocket, from_addr)):
         name = None
         bcc_addr = None
         if spr:
@@ -234,15 +238,18 @@
             bcc_addr)
 
 
-def assemble_body(blamer, spr, archive, distroseries, summary, changes,
+def assemble_body(blamer, spr, bprs, archive, distroseries, summary, changes,
                   action):
     """Assemble the e-mail notification body."""
+    if changes is None:
+        changes = {}
+    (changesfile, date, changedby, maintainer) = fetch_information(
+        spr, bprs, changes)
     information = {
         'STATUS': ACTION_DESCRIPTIONS[action],
         'SUMMARY': summary,
-        'DATE': 'Date: %s' % changes['Date'],
-        'CHANGESFILE': ChangesFile.formatChangesComment(
-            sanitize_string(changes.get('Changes'))),
+        'DATE': 'Date: %s' % date,
+        'CHANGESFILE': changesfile,
         'DISTRO': distroseries.distribution.title,
         'ANNOUNCE': 'No announcement sent',
         'CHANGEDBY': '',
@@ -254,7 +261,6 @@
         }
     if spr:
         information['SPR_URL'] = canonical_url(spr)
-    changedby = sanitize_string(changes.get('Changed-By'))
     if changedby:
         information['CHANGEDBY'] = '\nChanged-By: %s' % changedby
     origin = changes.get('Origin')
@@ -266,13 +272,11 @@
     if distroseries.changeslist:
         information['ANNOUNCE'] = "Announcing to %s" % (
             distroseries.changeslist)
-    if blamer is not None:
-        signer_signature = '%s <%s>' % (
-            blamer.displayname, blamer.preferredemail.email)
+    if blamer is not None and blamer != email_to_person(changedby):
+        signer_signature = person_to_email(blamer)
         if signer_signature != changedby:
             information['SIGNER'] = '\nSigned-By: %s' % signer_signature
     # Add maintainer if present and different from changed-by.
-    maintainer = sanitize_string(changes.get('Maintainer'))
     if maintainer and maintainer != changedby:
         information['MAINTAINER'] = '\nMaintainer: %s' % maintainer
     return get_template(archive, action) % information
@@ -409,11 +413,21 @@
         logger.debug(msg)
 
 
-def get_recipients(blamer, archive, distroseries, changes, logger):
+def get_recipients(blamer, archive, distroseries, logger, changes=None,
+                   spr=None, bprs=None):
     """Return a list of recipients for notification emails."""
     candidate_recipients = []
     debug(logger, "Building recipients list.")
-    changer = email_to_person(changes['Changed-By'])
+    (changesfile, date, changedby, maint) = fetch_information(
+        spr, bprs, changes)
+    if changedby:
+        changer = email_to_person(changedby)
+    else:
+        changer = None
+    if maint:
+        maintainer = email_to_person(maint)
+    else:
+        maintainer = None
 
     if blamer:
         # This is a signed upload.
@@ -434,7 +448,6 @@
 
     # If this is not a PPA, we also consider maintainer and changed-by.
     if blamer and not archive.is_ppa:
-        maintainer = email_to_person(changes['Maintainer'])
         if (maintainer and maintainer != blamer and
                 maintainer.isUploader(distroseries.distribution)):
             debug(logger, "Adding maintainer to recipients")
@@ -520,6 +533,12 @@
     return getUtility(IPersonSet).getByEmail(email)
 
 
+def person_to_email(person):
+    """Return a string of full name <e-mail address> given an IPerson."""
+    if person and person.preferredemail:
+        return format_address_for_person(person)
+
+
 def is_auto_sync_upload(spr, bprs, pocket, changed_by_email):
     """Return True if this is a (Debian) auto sync upload.
 
@@ -534,5 +553,24 @@
         pocket != PackagePublishingPocket.SECURITY)
 
 
+def fetch_information(spr, bprs, changes):
+    changedby = None
+    maintainer = None
+    if changes:
+        changesfile = ChangesFile.formatChangesComment(
+            sanitize_string(changes.get('Changes')))
+        date = changes.get('Date')
+        changedby = sanitize_string(changes.get('Changed-By'))
+        maintainer = sanitize_string(changes.get('Maintainer'))
+    elif spr or bprs:
+        if not spr and bprs:
+            spr = bprs[0].build.source_package_release
+        changesfile = spr.changelog_entry
+        date = spr.dateuploaded
+        changedby = person_to_email(spr.creator)
+        maintainer = person_to_email(spr.maintainer)
+    return (changesfile, date, changedby, maintainer)
+
+
 class LanguagePackEncountered(Exception):
     """Thrown when not wanting to email notifications for language packs."""

=== added file 'lib/lp/soyuz/adapters/tests/test_notification.py'
--- lib/lp/soyuz/adapters/tests/test_notification.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/adapters/tests/test_notification.py	2011-06-07 04:16:50 +0000
@@ -0,0 +1,130 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+from zope.security.proxy import removeSecurityProxy
+
+from canonical.testing.layers import LaunchpadZopelessLayer
+from lp.archivepublisher.utils import get_ppa_reference
+from lp.services.mail.sendmail import format_address_for_person
+from lp.soyuz.adapters.notification import (
+    calculate_subject,
+    fetch_information,
+    person_to_email,
+    notify,
+    )
+from lp.soyuz.enums import PackageUploadCustomFormat
+from lp.testing import TestCaseWithFactory
+from lp.testing.mail_helpers import pop_notifications
+
+
+class TestNotification(TestCaseWithFactory):
+
+    layer = LaunchpadZopelessLayer
+
+    def test_fetch_information_changes(self):
+        changes = {
+            'Date': '2001-01-01',
+            'Changed-By': 'Foo Bar <foo.bar@canonical.com>',
+            'Maintainer': 'Foo Bar <foo.bar@canonical.com>',
+            'Changes': ' * Foo!'
+            }
+        (changesfile, date, changedby, maintainer) = fetch_information(
+            None, None, changes)
+        self.assertEqual('2001-01-01', date)
+        self.assertEqual(' * Foo!', changesfile)
+        for field in (changedby, maintainer):
+            self.assertEqual('Foo Bar <foo.bar@canonical.com>', field)
+
+    def test_fetch_information_spr(self):
+        spr = self.factory.makeSourcePackageRelease()
+        (changesfile, date, changedby, maintainer) = fetch_information(
+            spr, None, None)
+        self.assertEqual(date, spr.dateuploaded)
+        self.assertEqual(changesfile, spr.changelog_entry)
+        self.assertEqual(changedby, format_address_for_person(spr.creator))
+        self.assertEqual(
+            maintainer, format_address_for_person(spr.maintainer))
+
+    def test_fetch_information_bprs(self):
+        bpr = self.factory.makeBinaryPackageRelease()
+        (changesfile, date, changedby, maintainer) = fetch_information(
+            None, [bpr], None)
+        spr = bpr.build.source_package_release
+        self.assertEqual(date, spr.dateuploaded)
+        self.assertEqual(changesfile, spr.changelog_entry)
+        self.assertEqual(changedby, format_address_for_person(spr.creator))
+        self.assertEqual(
+            maintainer, format_address_for_person(spr.maintainer))
+
+    def test_calculate_subject_spr(self):
+        spr = self.factory.makeSourcePackageRelease()
+        archive = self.factory.makeArchive()
+        pocket = self.factory.getAnyPocket()
+        distroseries = self.factory.makeDistroSeries()
+        expected_subject = '[PPA %s] [%s/%s] %s %s (Accepted)' % (
+            get_ppa_reference(archive), distroseries.distribution.name,
+            distroseries.getSuite(pocket), spr.name, spr.version)  
+        subject = calculate_subject(
+            spr, [], [], archive, distroseries, pocket, 'accepted')
+        self.assertEqual(expected_subject, subject)
+
+    def test_calculate_subject_bprs(self):
+        bpr = self.factory.makeBinaryPackageRelease()
+        archive = self.factory.makeArchive()
+        pocket = self.factory.getAnyPocket()
+        distroseries = self.factory.makeDistroSeries()
+        expected_subject = '[PPA %s] [%s/%s] %s %s (Accepted)' % (
+            get_ppa_reference(archive), distroseries.distribution.name,
+            distroseries.getSuite(pocket),
+            bpr.build.source_package_release.name, bpr.version)
+        subject = calculate_subject(
+            None, [bpr], [], archive, distroseries, pocket, 'accepted')
+        self.assertEqual(expected_subject, subject)
+
+    def test_calculate_subject_customfile(self):
+        lfa = self.factory.makeLibraryFileAlias()
+        package_upload = self.factory.makePackageUpload()
+        customfile = package_upload.addCustom(
+            lfa, PackageUploadCustomFormat.DEBIAN_INSTALLER)
+        archive = self.factory.makeArchive()
+        pocket = self.factory.getAnyPocket()
+        distroseries = self.factory.makeDistroSeries()
+        expected_subject = '[PPA %s] [%s/%s] %s - (Accepted)' % (
+            get_ppa_reference(archive), distroseries.distribution.name,
+            distroseries.getSuite(pocket), lfa.filename)
+        subject = calculate_subject(
+            None, [], [customfile], archive, distroseries, pocket,
+            'accepted')
+        self.assertEqual(expected_subject, subject)
+
+    def test_notify_bpr(self):
+        # If we notify about an accepted bpr with no source, it is from a
+        # build, and no notification is sent.
+        bpr = self.factory.makeBinaryPackageRelease()
+        archive = self.factory.makeArchive()
+        pocket = self.factory.getAnyPocket()
+        distroseries = self.factory.makeDistroSeries()
+        person = self.factory.makePerson()
+        notify(
+            person, None, [bpr], [], archive, distroseries, pocket,
+            action='accepted')
+        notifications = pop_notifications()
+        self.assertEqual(0, len(notifications))
+
+    def test_notify_bpr_rejected(self):
+        # If we notify about a rejected bpr with no source, a notification is
+        # sent.
+        bpr = self.factory.makeBinaryPackageRelease()
+        removeSecurityProxy(
+            bpr.build.source_package_release).changelog_entry = '* Foo!'
+        archive = self.factory.makeArchive()
+        pocket = self.factory.getAnyPocket()
+        distroseries = self.factory.makeDistroSeries()
+        person = self.factory.makePerson()
+        notify(
+            person, None, [bpr], [], archive, distroseries, pocket,
+            action='rejected')
+        [notification] = pop_notifications()
+        body = notification.as_string()
+        self.assertEqual(person_to_email(person), notification['To'])
+        self.assertIn('Rejected by archive administrator.\n\n* Foo!\n', body)

=== modified file 'lib/lp/soyuz/doc/soyuz-set-of-uploads.txt'
--- lib/lp/soyuz/doc/soyuz-set-of-uploads.txt	2011-05-30 19:35:09 +0000
+++ lib/lp/soyuz/doc/soyuz-set-of-uploads.txt	2011-06-07 23:33:42 +0000
@@ -332,7 +332,7 @@
   Date: Tue, 25 Apr 2006 10:36:14 -0300
   Changed-By: Celso R. Providelo <cprov@ubuntu.com>
   Maintainer: Launchpad team <launchpad@lists.canonical.com>
-  Signed-By: Foo B. Bar <foo.bar@canonical.com>
+  Signed-By: "Foo B. Bar" <foo.bar@canonical.com>
   http://launchpad.dev/ubuntutest/breezy/+source/bar/1.0-4
   <BLANKLINE>
   =3D=3D

=== modified file 'lib/lp/soyuz/scripts/packagecopier.py'
--- lib/lp/soyuz/scripts/packagecopier.py	2011-06-03 09:18:34 +0000
+++ lib/lp/soyuz/scripts/packagecopier.py	2011-06-07 23:33:42 +0000
@@ -28,6 +28,7 @@
 from canonical.librarian.utils import copy_and_close
 from lp.app.errors import NotFoundError
 from lp.buildmaster.enums import BuildStatus
+from lp.soyuz.adapters.notification import notify
 from lp.soyuz.adapters.packagelocation import build_package_location
 from lp.soyuz.enums import (
     ArchivePurpose,
@@ -509,7 +510,8 @@
 
 
 def do_copy(sources, archive, series, pocket, include_binaries=False,
-            allow_delayed_copies=True, person=None, check_permissions=True):
+            allow_delayed_copies=True, person=None, check_permissions=True,
+            send_email=False):
     """Perform the complete copy of the given sources incrementally.
 
     Verifies if each copy can be performed using `CopyChecker` and
@@ -534,6 +536,7 @@
     :param person: the requester `IPerson`.
     :param check_permissions: boolean indicating whether or not the
         requester's permissions to copy should be checked.
+    :param send_email: Should we notify for the copy performed?
 
     :raise CannotCopy when one or more copies were not allowed. The error
         will contain the reason why each copy was denied.
@@ -569,11 +572,18 @@
             destination_series = series
         if source.delayed:
             delayed_copy = _do_delayed_copy(
-                source, archive, destination_series, pocket, include_binaries)
+                source, archive, destination_series, pocket,
+                include_binaries)
             sub_copies = [delayed_copy]
         else:
             sub_copies = _do_direct_copy(
-                source, archive, destination_series, pocket, include_binaries)
+                source, archive, destination_series, pocket,
+                include_binaries)
+            if send_email:
+                notify(
+                    person, source.sourcepackagerelease, [], [], archive,
+                    destination_series, pocket, changes=None,
+                    action='accepted')
 
         copies.extend(sub_copies)
 

=== modified file 'lib/lp/soyuz/scripts/tests/test_copypackage.py'
--- lib/lp/soyuz/scripts/tests/test_copypackage.py	2011-05-25 03:32:48 +0000
+++ lib/lp/soyuz/scripts/tests/test_copypackage.py	2011-06-03 07:50:41 +0000
@@ -7,6 +7,7 @@
 import os
 import subprocess
 import sys
+from textwrap import dedent
 import unittest
 
 import pytz
@@ -28,6 +29,7 @@
     LaunchpadFunctionalLayer,
     LaunchpadZopelessLayer,
     )
+from lp.archivepublisher.utils import get_ppa_reference
 from lp.bugs.interfaces.bug import (
     CreateBugParams,
     IBugSet,
@@ -80,10 +82,13 @@
     )
 from lp.soyuz.tests.test_publishing import SoyuzTestPublisher
 from lp.testing import (
+    person_logged_in,
     StormStatementRecorder,
     TestCaseWithFactory,
     )
+from lp.testing.mail_helpers import pop_notifications
 from lp.testing.matchers import HasQueryCount
+from lp.testing.sampledata import ADMIN_EMAIL
 
 
 class ReUploadFileTestCase(TestCaseWithFactory):
@@ -1377,6 +1382,82 @@
         [copied_source] = self.doCopy(
             source, target_archive, dsp.derived_series, source.pocket, False)
 
+    def test_copy_ppa_generates_notification(self):
+        # When a copy into a PPA is performed, a notification is sent.
+        archive = self.test_publisher.ubuntutest.main_archive
+        source = self.test_publisher.getPubSource(
+            archive=archive, version='1.0-2', architecturehintlist='any')
+        source.sourcepackagerelease.changelog_entry = '* Foo!'
+        nobby = self.createNobby(('i386', 'hppa'))
+        getUtility(ISourcePackageFormatSelectionSet).add(
+            nobby, SourcePackageFormat.FORMAT_1_0)
+        target_archive = self.factory.makeArchive(
+            distribution=self.test_publisher.ubuntutest)
+        [copied_source] = do_copy(
+            [source], target_archive, nobby, source.pocket, False,
+            person=target_archive.owner, check_permissions=False,
+            send_email=True)
+        [notification] = pop_notifications()
+        self.assertEquals(
+            get_ppa_reference(target_archive),
+            notification['X-Launchpad-PPA'])
+        self.assertIn(
+            source.sourcepackagerelease.changelog_entry,
+            notification.as_string())
+
+    def test_copy_generates_notification(self):
+        # When a copy into a primary archive is performed, a notification is
+        # sent.
+        archive = self.test_publisher.ubuntutest.main_archive
+        source = self.test_publisher.getPubSource(
+            archive=archive, version='1.0-2', architecturehintlist='any')
+        source.sourcepackagerelease.changelog_entry = '* Foo!'
+        # Copying to a primary archive reads the changes to close bugs.
+        transaction.commit()
+        nobby = self.createNobby(('i386', 'hppa'))
+        getUtility(ISourcePackageFormatSelectionSet).add(
+            nobby, SourcePackageFormat.FORMAT_1_0)
+        admin = getUtility(IPersonSet).getByEmail(ADMIN_EMAIL)
+        nobby.changeslist = 'nobby-changes@example.com'
+        [copied_source] = do_copy(
+            [source], archive, nobby, source.pocket, False,
+            person=source.sourcepackagerelease.creator,
+            check_permissions=False, send_email=True)
+        [notification, announcement] = pop_notifications()
+        self.assertEquals(
+            'Foo Bar <foo.bar@canonical.com>', notification['To'])
+        self.assertEquals('nobby-changes@example.com', announcement['To'])
+        for mail in (notification, announcement):
+            self.assertEquals(
+                '[ubuntutest/nobby] foo 1.0-2 (Accepted)', mail['Subject'])
+        expected_text = dedent("""
+            * Foo!
+
+            Date: %s
+            Changed-By: Foo Bar <foo.bar@canonical.com>
+            http://launchpad.dev/ubuntutest/breezy-autotest/+source/foo/1.0-2
+            """ % source.sourcepackagerelease.dateuploaded)
+        self.assertIn(expected_text, notification.as_string())
+        self.assertIn(expected_text, announcement.as_string())
+
+    def test_copy_does_not_generate_notification(self):
+        # When notify = False is passed to do_copy, no notification is
+        # generated.
+        archive = self.test_publisher.ubuntutest.main_archive
+        source = self.test_publisher.getPubSource(
+            archive=archive, version='1.0-2', architecturehintlist='any')
+        source.sourcepackagerelease.changelog_entry = '* Foo!'
+        nobby = self.createNobby(('i386', 'hppa'))
+        getUtility(ISourcePackageFormatSelectionSet).add(
+            nobby, SourcePackageFormat.FORMAT_1_0)
+        target_archive = self.factory.makeArchive(
+            distribution=self.test_publisher.ubuntutest)
+        [copied_source] = do_copy(
+            [source], target_archive, nobby, source.pocket, False,
+            person=target_archive.owner, check_permissions=False,
+            send_email=False)
+        self.assertEquals([], pop_notifications())
+
 
 class TestDoDelayedCopy(TestCaseWithFactory, BaseDoCopyTests):
 

